// access.cpp:
//
// show security information for a given file

#include <stdio.h>
#include <string.h>
#include "security.hpp"

void PrintAcl(AccessControlList* acl)
{
  AccessControlEntry ace;
  unsigned int acl_length = acl->GetLength();

  for (unsigned int acl_index = 0; acl_length; acl_index++) {
    if (!acl->GetAce(acl_index, ace)) {
      if (acl_index == 0)
	printf("<empty>\n");
      return;
    }

    char* acl_type;

    switch (ace.GetType()) {
    case ACCESS_ALLOWED_ACE_TYPE:
      acl_type = "allow";
      break;
    case ACCESS_DENIED_ACE_TYPE:
      acl_type = "deny";
      break;
    default:
      // don't care
      continue;
    }

    printf("    %s %s=(", ace.GetAccount(), acl_type);

    unsigned int first = 1;
    unsigned int acl_mask = ace.GetMask();

    // file attributes
    if (acl_mask & FILE_READ_DATA) {
      printf("read");
      first = 0;
    }
    if (acl_mask & FILE_WRITE_DATA) {
      printf("%swrite", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & FILE_APPEND_DATA) {
      printf("%sappend", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & FILE_EXECUTE) {
      printf("%sexecute", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & FILE_DELETE_CHILD) {
      printf("%sdelete_child", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & FILE_READ_ATTRIBUTES) {
      printf("%sread_attributes", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & FILE_WRITE_ATTRIBUTES) {
      printf("%swrite_attributes", (first ? "" : ","));
      first = 0;
    }

    // generic attributes
    if (acl_mask & DELETE) {
      printf("%sdelete", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & READ_CONTROL) {
      printf("%sacl_read", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & WRITE_DAC) {
      printf("%sacl_write", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & WRITE_OWNER) {
      printf("%schange_owner", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & SYNCHRONIZE) {
      printf("%ssynchronize", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & GENERIC_ALL) {
      printf("%sgeneric_all", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & GENERIC_EXECUTE) {
      printf("%sgeneric_execute", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & GENERIC_WRITE) {
      printf("%sgeneric_write", (first ? "" : ","));
      first = 0;
    }
    if (acl_mask & GENERIC_READ)
      printf("%sgeneric_read", (first ? "" : ","));

    printf(")");

    unsigned int flags = ace.GetFlags();

    if (flags != 0) {
      first = 1;

      printf(" flags=(");

      if (flags & CONTAINER_INHERIT_ACE) {
	printf("container_inherit");
	first = 0;
      }
      if (flags & OBJECT_INHERIT_ACE) {
	printf("%sobject_inherit", (first ? "" : ","));
	first = 0;
      }
      if (flags & INHERIT_ONLY_ACE) {
	printf("%sinherit_only", (first ? "" : ","));
	first = 0;
      }
      if (flags & NO_PROPAGATE_INHERIT_ACE) {
	printf("%sno_propagate", (first ? "" : ","));
	first = 0;
      }
      if (flags & FAILED_ACCESS_ACE_FLAG) {
	printf("%sfailed_flag", (first ? "" : ","));
	first = 0;
      }
      if (flags & SUCCESSFUL_ACCESS_ACE_FLAG)
	printf("%saccess_flag", (first ? "" : ","));

      printf(")");
    }

    printf("\n");
  }
}

void PrintAccess(LPTSTR filename)
{
  SecurityDescriptor *sd = GetFileSecurityDescriptor(filename);
  if (sd == NULL) {
    fprintf(stderr, "%s: %s\n", filename, strerror(GetLastError()));
    return;
  }

  printf("  owner = %s\n", sd->GetOwner()->GetAccount());
  printf("  group = %s\n", sd->GetGroup()->GetAccount());

  AccessControlList* acl = sd->GetSystemAcl();
  if (acl != NULL) {
    printf("  system access control list:\n");
    PrintAcl(acl);
  }

  acl = sd->GetDiscretionaryAcl();
  if (acl != NULL) {
    printf("  discretionary access control list:\n");
    PrintAcl(acl);
  }

  delete sd;
}



void access(LPTSTR fileName)
{
	PrintAccess(fileName);
  
}

